// Fecha.java

/**
 * Conjunto de mtodos para el tratamiento de fechas.
 * @author  Luis Hernndez
 * @version 1.0
 */
public class Fecha {
  static int dia, mes, anio;

  /** Programa principal. */
  public static void main(String[] args)
  {
    leer();
    mostrarCorta();
    System.out.println("Mes con " + diasMes(mes) + " dias.");
    mostrarLarga();
    if(bisiesto()) System.out.println("Es un ao bisiesto");
    else System.out.println("No es un ao bisiesto");
    long dias = diasTranscurridos();
    System.out.println(dias + " dias desde el 1-1-1900");
    fechaTras(dias);
    System.out.print("Despues de pasar " + dias
      + " dias desde el 1-1-1900 se llega al ");
    mostrarCorta();
    siguiente();
    mostrarLarga();
  }

  /** Lectura de la fecha (da, mes y ao). */
  public static void leer() {
    System.out.print("Dia: ");
    dia = MyInput.readInt();
    System.out.print("Mes: ");
    mes = MyInput.readInt();
    System.out.print("Ao: ");
    anio = MyInput.readInt();
    valida();
  }

  /** Das del mes de la fecha.
   * @param  mes El mes del que se quiere saber su nmero de das. */
  public static int diasMes(int mes) {
    int numeroDias;
    switch(mes) {
      case 1: case 3: case 5:
      case 7: case 8: case 10:
      case 12:
        numeroDias = 31;
        break;
      case 4: case 6: case 9:
      case 11:
        numeroDias = 30;
        break;
      default:
        if(bisiesto()) numeroDias = 29;
        else numeroDias = 28;
    }
    return numeroDias;
  }

  /** Comprueba si la fecha es correcta y si no la ajusta.
   * Se llama tras la lectura de los datos. */
  public static void valida() {
    if((anio < 1900) || (anio > 2050)) anio = 1900;
    if((mes < 1) || (mes > 12)) mes = 1;
    if(dia < 1) dia = 1;
    else {
      if(mes == 2) {
        if(bisiesto()) { if(dia > 29) dia = 1; }
        else if(dia > 28) dia = 1;
      }
      else if(dia > diasMes(mes)) dia = 1;
    }
  }

  /** Indica si el ao es bisiesto o no. */
  public static boolean bisiesto() {
    return (anio % 4) == 0;
  }

  /** Da de la semana de la fecha (0 para domingo). */
  public static int diaSemana() {
    int nd = (int) (diasTranscurridos() % 7); // 0 para domingo
    return nd;
  }

  /** Muestra la fecha en formato largo (p.e. martes 2 septiembre de 2003). */
  public static void mostrarLarga() { // martes 2 de septiembre de 2003
    int ds = diaSemana();
    switch(ds) {
      case 1: System.out.print("lunes"); break;
      case 2: System.out.print("martes"); break;
      case 3: System.out.print("miercoles"); break;
      case 4: System.out.print("jueves"); break;
      case 5: System.out.print("viernes"); break;
      case 6: System.out.print("sabado"); break;
      default: System.out.print("domingo");
    }
    System.out.print(" " + dia + " de ");
    switch(mes) {
      case 1: System.out.print("enero"); break;
      case 2: System.out.print("febrero"); break;
      case 3: System.out.print("marzo"); break;
      case 4: System.out.print("abril"); break;
      case 5: System.out.print("mayo"); break;
      case 6: System.out.print("junio"); break;
      case 7: System.out.print("julio"); break;
      case 8: System.out.print("agosto"); break;
      case 9: System.out.print("septiembre"); break;
      case 10: System.out.print("octubre"); break;
      case 11: System.out.print("noviembre"); break;
      case 12: System.out.print("diciembre"); break;
    }
    System.out.println(" de " + anio);
  }

  /** Muestra la fecha en formato corto (p.e. 02-09-2003). */
  public static void mostrarCorta() { // 02-09-2003
    if(dia < 10) System.out.print("0");
    System.out.print(dia + "-");
    if(mes < 10) System.out.print("0");
    System.out.println(mes + "-" + anio);
  }

  /** Devuelve los das transcurridos desde el 1-1-1900 hasta la fecha. */
  public static long diasTranscurridos() { // desde el 1 de enero de 1900
    long total;
    int anios, m, bisiestos = 0;
    anios = anio - 1900;
    total = anios * 365;
    bisiestos = (anios + 3) / 4;
    total += bisiestos;
    for(m = 1; m < mes; m++) total += diasMes(m);
    total += dia; // Total de das desde 1-1-1900 hasta la fecha.
    // Incluyendo ambas fechas (se ha de restar una).
    total--;
    return total;
  }

  /** Establece la fecha que corresponde tras transcurrir
   * los das que se proporcionan desde el 1-1-1900. */
  public static void fechaTras(long dias) { // 1 de enero de 1900 ms esos das
    int diasAnio, diasMes;
    dia = 1;
    mes = 1;
    anio = 1900;
    if(dias < 0) return;
    while(dias > 0) {
      if(bisiesto()) diasAnio = 366;
      else diasAnio = 365;
      if(dias >= diasAnio) {
        dias -= diasAnio;
        anio++;
        if(anio == 2051) {
          dia = 31;
          mes = 12;
          anio = 2050;
          return;
        }
      }
      else {
        if(dias >= diasMes(mes)) {
          dias -= diasMes(mes);
          mes++;
        }
        else {
          dia += dias;
          dias = 0;
        }
      }
    }
  }

  /** Pasa al da siguiente. */
  public static void siguiente() {
    long dias = diasTranscurridos();
    fechaTras(dias + 1);
  }
}
