// Racional.java: manejo de nmeros racionales (fracciones)

public class Racional {
  // Atributos: numerador y denominador
  private long _numerador;
  private long _denominador;

  public Racional() { // Constructor predeterminado: 0/1
    this(0, 1);
  }

  private void simplifica() { // Simplifica la fraccin
  // Algoritmo de Euclides para el clculo del Mximo Comn Divisor (MCD)
  //   Primer dividendo: el valor mayor de entre el numerador y el denominador
  //   Primer divisor: el valor menor de entre el numerador y el denominador
  //   Repetir sucesivas divisiones enteras entre dividendo y divisor
  //   Cada siguiente divisin: como dividendo, el divisor anterior
  //                            como divisor, el resto de la divisin anterior
  //   Parar al obtener 0 como resto
  //   El MCD es el divisor de la ltima divisin
  // Simplificacin de la fraccin: dividir numerador y denominador por el MCD
    long dividendo, divisor, cociente, resto;

    if((_numerador == 0) || (_denominador == 0)) return;
    if(_numerador > _denominador) {
      dividendo = _numerador;
      divisor = _denominador;
    }
    else {
      dividendo = _denominador;
      divisor = _numerador;
    }
    cociente = dividendo / divisor;
    resto = dividendo % divisor;
    while(resto != 0) {
      dividendo = divisor;
      divisor = resto;
      cociente = dividendo / divisor;
      resto = dividendo % divisor;
    }
    _numerador /= divisor;
    _denominador /= divisor;
  }

  public Racional(long numerador, long denominador) {
    _numerador = numerador;
    _denominador = denominador;
    simplifica();
  }

  // Accedentes (no necesitamos mutadores)

  public long numerador() { return _numerador; }

  public long denominador() { return _denominador; }

  // Lectura
  public void leer() {
    System.out.print("Numerador: ");
    _numerador = MyInput.readLong();
    System.out.print("Denominador: ");
    _denominador = MyInput.readLong();
    simplifica();
  }

  // Operaciones aritmticas

  public Racional mas(Racional otro) {
    long n = _numerador * otro._denominador +
      _denominador * otro._numerador;
    long d = _denominador * otro._denominador;
    Racional resultado = new Racional(n, d);
    resultado.simplifica();
    return resultado;
  }

  public Racional menos(Racional otro) {
    long n = _numerador * otro._denominador -
      _denominador * otro._numerador;
    long d = _denominador * otro._denominador;
    Racional resultado = new Racional(n, d);
    resultado.simplifica();
    return resultado;
  }

  public Racional por(Racional otro) {
    long n = _numerador * otro._numerador;
    long d = _denominador * otro._denominador;
    Racional resultado = new Racional(n, d);
    resultado.simplifica();
    return resultado;
  }

  public Racional entre(Racional otro) {
    long n = _numerador * otro._denominador;
    long d = _denominador * otro._numerador;
    Racional resultado = new Racional(n, d);
    resultado.simplifica();
    return resultado;
  }

  // Operaciones relacionales

  public boolean igualQue(Racional otro) {
    return (_numerador * otro._denominador) ==
      (_denominador * otro._numerador);
  }

  public boolean menorQue(Racional otro) {
    float izq, der;
    izq = (float) _numerador / (float) _denominador;
    der = (float) otro._numerador / (float) otro._denominador;
    return izq < der;
  }

  public boolean mayorQue(Racional otro) {
    float izq, der;
    izq = (float) _numerador / (float) _denominador;
    der = (float) otro._numerador / (float) otro._denominador;
    return izq > der;
  }

  // Clonacin

  public Racional copia() {
    return new Racional(_numerador, _denominador);
  }

  public void print() {
    System.out.print(_numerador + " / " + _denominador);
  }
}

